﻿#if !defined( BASEMESH_H )
#define BASEMESH_H
/* Shaderprogramozás - Grafika és játékfejlesztés DirectX-szel ----------------

	Állomány          : BaseMesh.h

    Nyelv             : C++

    Létrehozás dátuma : 2006.10.17

    Utolsó módosítás  : 2009.03.15

    Leírás            : Shaderprogramozás
						Grafika és játékfejlesztés DirectX-szel
						11.6.1. Általános háttérmegvilágítás
						324. oldal
	3D-modell kezelést megvalósító ősosztály
	
               Copyright (C) Nyisztor Károly 2006-2009
							www.directx.hu
-----------------------------------------------------------------------------*/

/*- Befordított fejállományok -----------------------------------------------*/
#include <d3d9.h>
#include <d3dx9.h>

#include <string>
using std::string;

/*- Névterek ----------------------------------------------------------------*/
namespace app
{
namespace scene
{
/*- Definíciók és konstansok ------------------------------------------------*/

/*- Saját típusok és osztályok -----------------------------------------------*/
/// <summary>
/// BaseMesh - a külső fájlból betöltött 3D-modell kezelést megvalósító osztályok ősosztálya
/// </summary>
///<remarks>
///A BaseFileLoader ősosztály definiálja az alapvető függvényeket
///Referenciaszámlálással és a FlyweightFactory osztály segítségével biztosítjuk, hogy fájlonként 
///egy erőforrás jöjjön létre, amin osztoznak a kliensek
/// </remarks>
class BaseMesh
{	
// adattagok
protected:	
	///<summary>
	///erőforráshoz tartozó fájlnév
	///</summary>
	const string myFileName;
	/// <summary>
	/// pointer a DirectX9 kompatibilis eszköz interfészre
	/// </summary>
	///<remarks>
	///static - minden példány ezen osztozkodik
	///</remarks>
	static LPDIRECT3DDEVICE9  myD3DDevicePtr;
	///<summary>
	///mutató az ID3DXMESH interfészre
	///</summary>	
	LPD3DXMESH myXMeshPtr;
	///<summary>
	///X-állományból betöltött anyagok
	///</summary>
	D3DMATERIAL9* myMeshMaterialsPtr;
	///<summary>
	///X-állományból betöltött textúrákat tároló puffer
	///</summary>
	LPDIRECT3DTEXTURE9* myMeshTexturesPtr;
	///<summary>
	///anyagok száma
	///</summary>
	DWORD myMaterialCnt;
	///<summary>
	///felületek felsőértéke
	///</summary>
	DWORD myMaxFaces;
	///<summary>
	///mutató a textúra interfészre
	///</summary>
	LPDIRECT3DTEXTURE9 myTexturePtr;

// tagfüggvények
public:
	//elrejtjük a konstruktorokat, a másoló ctor-t, és az értékadó operátor,
	//hogy közvetlenül ne lehessen példányosítani 
	BaseMesh( const string& fileName_in );
	///<summary>
	///default ctor
	///</summary>
	BaseMesh(){};

	// destruktor
	virtual ~BaseMesh();

	virtual void Release();

	virtual HRESULT render();

	virtual HRESULT setVertexDecl( const D3DVERTEXELEMENT9 *newDecl, 
                           bool computeNormalsFlag = true, 
						   bool computeTangentsFlag = true, 
                           bool bSplitVertexForOptimalTangents = false );

	virtual HRESULT render( ID3DXEffect* effectPtr,
					D3DXHANDLE hTexture = NULL,
					D3DXHANDLE hMat = NULL,
					bool bDrawOpaqueSubsets = false,
					bool bDrawAlphaSubsets = false );
private:
	HRESULT init();
};

} // namespace scene
} // namespace game

#endif