#if !defined( CAMERA_H )
#define CAMERA_H
/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : Camera.h

    Nyelv             : C++

    Ltrehozs dtuma : 2005.07.07

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.1.9. "Kamerakezels"
						295. oldal
	Ez az llomny a Camera osztly interfszt tartalmazza
	
               Copyright (C) Nyisztor Kroly 2005-2009
							www.directx.hu
-----------------------------------------------------------------------------*/
/*- Befordtott fejllomnyok -----------------------------------------------*/
#include <d3dx9.h>

#include "..\input\CInput.h"
using namespace app::input;

/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
namespace scene
{
	class app::input::CInput;
/*- Sajt tpusok s osztlyok -----------------------------------------------*/
class Camera
{
// adattagok
private:
	// D3DDevice mutat
	LPDIRECT3DDEVICE9  myD3DDevicePtr;
	CInput* myInputPtr;
	
	// kamera helyzetvektora
	D3DXVECTOR3 myPosition;
	// a pont helyvektora, amire a kamera irnyul
	D3DXVECTOR3 myLookAtTarget;
	// kamerhoz viszonytott felfele irny
	D3DXVECTOR3 myUp;

	D3DXMATRIX myViewMatrix;
	D3DXMATRIX myProjectionMatrix;

	D3DXMATRIX myOrthoMatrix;

	D3DXMATRIX myRotMat;

	D3DXMATRIX myOrientationMatrix;
	// kpkocknknt kiszmtott forgatsi kvaterni
	D3DXQUATERNION myFrameRotQuat;
	// kamera forgatsi kvaterni (kumullt)
	D3DXQUATERNION myCameraQuat;

	float myCameraMoveX;
	float myCameraMoveY;
	float myCameraMoveZ;

	float myCameraRotX;
	float myCameraRotY;

	float myRotYaw;
	float myRotPitch;
	float myRotRoll;	

// tagfggvnyek
public:
	// konstruktor
	Camera();	
	// destruktor
	~Camera();

	void update( const float dt_in = 0.f );

	// kamera helyzett meghatroz mtrix belltsa
	void setupViewMatrix( D3DXVECTOR3& posVec_in, 
						  D3DXVECTOR3& lookAtTarget_in, 
						  D3DXVECTOR3& up_in );

	// projekcis mtrix belltsa
	void setupProjectionMatrix( float fov_in, float aspectRatio_in, float nearPlane_in, float farPlane_in );

	D3DXMATRIX getViewMatrix()
	{
		return myViewMatrix;
	}
	D3DXMATRIX getProjMatrix()
	{
		return myProjectionMatrix;
	}

	D3DXMATRIX getOrthoMatrix()
	{
		return myOrthoMatrix;
	}

	D3DXMATRIX getViewProjMatrix()
	{
		return (myViewMatrix * myProjectionMatrix);
	}
	

	D3DXVECTOR3 getPosition()
	{
		return myPosition;
	}

	D3DXVECTOR3 getYPRVec3()
	{
		return D3DXVECTOR3( myRotYaw, myRotPitch, myRotRoll );
	}

	const D3DXMATRIX* getRotMat()
	{
		D3DXMatrixRotationYawPitchRoll(  &myRotMat, myRotYaw, myRotPitch, myRotRoll );
		return &myRotMat;
	}	

	D3DXVECTOR3 getDir();	


private:
	// kamera mozgatsa s forgatsa
	void move( D3DXVECTOR3& positionDelta_in = D3DXVECTOR3( 0, 0, 0 ), 
			   const D3DXVECTOR3& rotYPRdelta_in = D3DXVECTOR3( 0, 0, 0 ), const float dt_in = 0.f );

	void setupProjection();
	void setupView();
};

} // namespace scene
} // namespace game

#endif