#if !defined( ERRORLOGGER_H )
#define ERRORLOGGER_H
/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : ErrorLogger.h

    Nyelv             : C++

    Ltrehozs dtuma : 2006.10.17

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.1.1.  "Hibakezels, naplzs"
						256. oldal
	
	Ez az llomny a hibanaplzst megvalst osztly interfszt tartalmazza
	
               Copyright (C) Nyisztor Kroly 2006-2009
							www.directx.hu
-----------------------------------------------------------------------------*/
/*- Befordtott fejllomnyok -----------------------------------------------*/
#include <sstream>
#include <vector>
#include <windows.h>
#include <iosfwd>
#include <fstream>
#include <string>
#include <iomanip>
/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
namespace debug
{

/*- Defincik s konstansok ------------------------------------------------*/
// hasznos makrk a hibanaplz osztly hasznlatra
/*!
 * makr a singleton osztly hasznlatra 
 */
#define ERRORLOGGER *(ErrorLogger::Instance())
/*!
 * makr zenetek kiratsra, amely beszrja a fjl nevt s a sor szmt
 * \param str_in a felhasznl ltal megadott szveg
 */
#define LOG_INFO( str_in ) \
	ERRORLOGGER << "[-]" <<  str_in << "[" << __FILE__ << "(" << __LINE__ << ")" << "]" << "";

#define LOG_WARNING( str_in ) \
	ERRORLOGGER << "[?]" <<  str_in << "[" << __FILE__ << "(" << __LINE__ << ")" << "]" << "";

#define LOG_ERROR( str_in ) \
	ERRORLOGGER << "[!]" <<  str_in << "[" << __FILE__ << "(" << __LINE__ << ")" << "]" << "";

#define LOG_EXCEPTION( errorStr_in ) \
	ERRORLOGGER << "[!]" <<  errorStr_in << "";
/*!
 * makr a naplzs bekapcsolsra
 */
#define ERRLOG_ON ErrorLogger::enable()
/*!
 * makr a naplzs kikapcsolsra
 */
#define ERRLOG_OFF ErrorLogger::disable()
/*!
 * \brief
 * UM_SETSCROLL sajt zenet definilsa
 * ezzel az zenettel rtestjk a konzolt, ha frisstsre van szksg
 */
const int UM_SETSCROLL = WM_USER + 32;
/*- Sajt tpusok s osztlyok -----------------------------------------------*/
class ErrorLogger
{
// adattagok
private:	
	///<summary>
	/// ablakkezel
	///</summary>
	static HWND		myHWND;  	
	///<summary>
	///(hiba)zeneteket tartalmaz puffer
	///</summary>
	///<remarks>
	///string-bejegyzseket tartalmaz tmb
	///</remarks>
	static std::vector<std::string> myStrBuffer;  
	///<summary>
	///az zeneteket trol puffer maximlis mrete (sorok szma)
	///</summary>
	///<remarks>
	///a mret tllpskor kirjuk a tartalmt a napl-llomnyba
	///</remarks>
	static unsigned int		myMaxBufferSize;
	///<summary>
	/// sor vgt jelz flag
	///</summary>
	///<remarks>
	/// ha rtke igaz, akkor a kvetkez karakterfzr j elemknt kerl a myStrBuffer vectorba,
	/// ellenkez esetben hozzadjuk az adott indexen tallhat stringhez
	///</remarks>
	static bool		myLineEndFlag;
	///<summary>
	///kln kozol ablak, vagy az alkalmazs ablaka
	///</summary>
	///<remarks>
	///igaz rtk esetn j konzolablak jn ltre az zenetek megjelentsre
	///</remarks>
	static bool		mySeparateConsole;
	///<summary>
	///konzol ablak teteje
	///</summary>
	static int		myPosTop;
	///<summary>
	///konzol ablak bal szle
	///</summary>
	static int		myPosLeft;
	///<summary>
	///ablak megsznst jelz flag
	///</summary>
	static bool		myDestroyedFlag;	
	///<summary>
	///konzol llapott tkrz flag
	///</summary>
	///<remarks>
	///ha hamis, nem runk a konzolra
	///</remarks>
	static bool		myActiveFlag;
	//default logging file
	///<summary>
	///a naplllomny neve
	///</summary>
	static std::ofstream myLogFile;

	///<summary>
	///trtnjen llomnyba naplzs?
	///</summary>
	static bool myLogToFile;
	///<summary>
	///naplzs a konzol-ablakba?
	///</summary>		
	static bool myLogToConsole;

	///<summary>
	///konzol ablak szlessge
	///</summary>
	static int myWindowWidth;
	///<summary>
	///konzol ablak magassga
	///</summary>
	static int myWindowHeight;

// tagfggvnyek
private:

	///<summary>
	///ctor
	///</summary>
	ErrorLogger(){}
 
	//privt copy ctor s rtkad opertor a kzvetlen pldnyosts elkerlsre
	ErrorLogger( const ErrorLogger& );
	ErrorLogger& operator=( const ErrorLogger& );

	void static initLogFile();
	static bool initConsole();

	static LRESULT CALLBACK LogWindowProc( HWND hwnd_in, UINT msg_in, WPARAM wparam_in, LPARAM lparam_in );		

	static void drawWindow()
	{
		InvalidateRect(myHWND, NULL, TRUE); 
		UpdateWindow(myHWND);
	}


public:
	static ErrorLogger* Instance();

	~ErrorLogger();

	///<summary>
	///felldefinilt operator << (sablonfggvny)
	///</summary>
	///<param name=t>sablon paramter</param>
	///<returns>visszaad egy referencit az ErrorLogger pldnyra</returns>
	///<remarks>
	/// a "" lezr karakternek szmt
	///</remarks>
	template <class T>
	ErrorLogger& operator<<( const T& t_in )
	{	
		// hibakezels arra az esetre, ha kikapcsoltk a loggolst, 
		// vagy megsznt az ablak
		if ( !myActiveFlag || myDestroyedFlag ) 
		{
			return *this;
		}
   
		// a puffer mrete elrte a fels rtket
		// kirjuk a tartalmt egy fjlba s kipucoljuk
		if ( myStrBuffer.size() > myMaxBufferSize )
		{
			commitBuffer();
		}
    
		std::ostringstream aStrStream; 
		aStrStream << t_in;
		
		// lezrult a string?
		if ( aStrStream.str() == "" )
		{
			flush(); 
			return *this;
		}

		// hozzadjuk a kvetkez string-et a vector adott indexn tallhat stringhez
		if ( !myLineEndFlag )
		{
				myStrBuffer.back() += aStrStream.str();
		}
		// vget rt az elz sor -> j string-bejegyzs kerl a vectorba
		else
		{
			myStrBuffer.push_back( aStrStream.str() );
			myLineEndFlag = false;
		}
		return *this;
  }
	///<summary>
	///a bels puffer kirtse
	///</summary>
	void clear()
	{ 
		myStrBuffer.clear(); 
		flush(); 
	}
	///<summary>
	///feldolgozza a lezrt zenetet
	///</summary>
	static void flush()
	{
		// jelezzk, hogy a kvetkez mr egy j bejegyzs
		myLineEndFlag = true; 
		// elkldjk az zenetet a konzolnak
		if ( !myDestroyedFlag && myLogToConsole )
		{			
			SendMessage( myHWND, UM_SETSCROLL, NULL, NULL );
		}
	}

	void commitBuffer();

	///<summary>
	///naplzs letiltsa
	///</summary>
	static void  disable(){ myActiveFlag = false; }
	///<summary>
	///naplzs engedlyezse
	///</summary>
	static void  enable() { myActiveFlag = true; }

	///<summary>
	///lekrdezi a konzol ablak llapott
	///</summary>
	///<returns>true, ha mr megsznt, false, ha mg ltezik</returns>
	bool isDestroyed() const{ return myDestroyedFlag; }
};

} // namespace debug
} // namespace game

#endif
